"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const util_1 = require("../util");
class Validator {
    constructor(errors = {}, processing = false, successful = false) {
        this.errors = errors;
        this.processing = processing;
        this.successful = successful;
    }
    add(field, message, forceUpdate) {
        if (forceUpdate || this.missed(field)) {
            this.errors[field] = [message];
        }
        else if (!this.errors[field].includes(message)) {
            this.errors[field].unshift(message);
        }
    }
    has(field) {
        const fields = this.fields(field);
        return (0, util_1.is)(Object.keys(this.errors), fields);
    }
    first(field) {
        const fields = this.fields((0, lodash_1.castArray)(field));
        const foundField = fields.find((f) => (0, lodash_1.has)(this.errors, f)) || '';
        const value = this.get(foundField);
        return Array.isArray(value) ? value[0] : value;
    }
    firstBy(obj, field = Object.keys(obj)[0]) {
        return (0, lodash_1.castArray)(obj[field])[0];
    }
    missed(field) {
        return !this.has(field);
    }
    nullState(field) {
        return this.has(field) ? this.missed(field) : null;
    }
    any(field = [], returnObject) {
        const fields = this.fields(field);
        const errors = {};
        if (!fields.length)
            return returnObject ? {} : Object.keys(this.errors).length > 0;
        fields.forEach((f) => {
            const val = this.get(f);
            if (returnObject && val.length)
                errors[f] = val;
            else if (!returnObject)
                errors[f] = val;
        });
        return returnObject ? errors : Object.keys(errors).length > 0;
    }
    get(field) {
        return (0, lodash_1.get)(this.errors, field, []);
    }
    all() {
        return this.errors;
    }
    count() {
        return Object.keys(this.errors).length;
    }
    fill(errors) {
        this.errors = errors;
    }
    flush() {
        this.fill({});
    }
    clear(field) {
        if (!field)
            return this.flush();
        const errors = (0, lodash_1.omit)((0, lodash_1.cloneDeep)(this.errors), this.fields(field));
        this.fill(errors);
    }
    isValid() {
        return !this.any();
    }
    onKeydown(event, prefix) {
        const { name } = event.target;
        if (!name)
            return;
        const names = prefix ? [name, `${prefix}.${name}`] : [name];
        this.clear(names);
    }
    fields(field) {
        const processField = (f) => {
            if (f.includes('*')) {
                const regex = new RegExp(`^${(0, lodash_1.replace)(f, '*', '.*')}$`, 'i');
                for (const key in this.errors) {
                    if (regex.test(key))
                        fields.push((0, util_1.toCamelCase)(key), (0, util_1.toSnakeCase)(key));
                }
            }
            else
                fields.push((0, util_1.toCamelCase)(f), (0, util_1.toSnakeCase)(f));
        };
        const fields = [];
        Array.isArray(field) ? field.forEach(processField) : processField(field);
        return [...new Set(fields)].filter(Boolean);
    }
}
exports.default = new Validator();
